import datetime

from sqlalchemy import create_engine, MetaData, Table, Column, Integer,\
    String, DateTime, ForeignKey, Text
from sqlalchemy.orm import mapper, sessionmaker
from sqlalchemy.sql import default_comparator


class ServerStorage:
    class AllUser:
        """Класс - отображение таблицы все пользователей"""
        def __init__(self, username, password_hash):
            self.name = username
            self.last_login = datetime.datetime.now()
            self.id = None
            self.password_hash = password_hash
            self.public_key = None

    class ActiveUser:
        """Класс- отображение таблицы всех активных пользователей"""
        def __init__(self, user_id, ip_address, port, at_login_time):
            self.id = None
            self.user = user_id
            self.ip_address = ip_address
            self.port = port
            self.at_login_time = at_login_time

    class LoginHistory:
        def __init__(self, username, date, ip, port):
            self.id = None
            self.name = username
            self.ip = ip
            self.port = port
            self.date = date

    class UserContacts:
        def __init__(self, user, contact):
            self.id = None
            self.user = user
            self.contact = contact

    class UserHistory:
        def __init__(self, user):
            self.id = None
            self.user = user
            self.sent = 0
            self.accepted = 0

    def __init__(self, path):
        server_database = f'sqlite:///{path}'
        self.database_engine = create_engine(
            server_database,
            echo=False,
            pool_recycle=7200,
            connect_args={
                'check_same_thread': False
            }
        )
        self.metadata = MetaData()

        users_table = Table('Users', self.metadata,
                            Column('id', Integer, primary_key=True),
                            Column('name', String, unique=True),
                            Column('last_login', DateTime),
                            Column('password_hash', String),
                            Column('public_key', Text),
                            )
        active_users_table = Table('Active_users', self.metadata,
                                   Column('id', Integer, primary_key=True),
                                   Column('user', ForeignKey('Users.id'),
                                          unique=True),
                                   Column('ip_address', String),
                                   Column('port', Integer),
                                   Column('at_login_time', DateTime),
                                   )
        login_history_table = Table('Login_history', self.metadata,
                                    Column('id', Integer, primary_key=True),
                                    Column('user', ForeignKey('Users.id'),
                                           unique=True),
                                    Column('ip_address', String),
                                    Column('port', Integer),
                                    Column('date', DateTime),
                                    )
        contacts_table = Table('Contacts', self.metadata,
                               Column('id', Integer, primary_key=True),
                               Column('user', ForeignKey('Users.id')),
                               Column('contact', ForeignKey('Users.id')),
                               )
        user_history_table = Table('User_history', self.metadata,
                                   Column('id', Integer, primary_key=True),
                                   Column('user', ForeignKey('Users.id')),
                                   Column('sent', Integer),
                                   Column('accepted', Integer),
                                   )

        self.metadata.create_all(self.database_engine)
        mapper(self.AllUser, users_table)
        mapper(self.ActiveUser, active_users_table)
        mapper(self.LoginHistory, login_history_table)
        mapper(self.UserContacts, contacts_table)
        mapper(self.UserHistory, user_history_table)

        Session = sessionmaker(bind=self.database_engine)
        self.session = Session()

        self.session.query(self.ActiveUser).delete()
        self.session.commit()

    def user_login(self, username, ip_address, port, key):
        result = self.session.query(self.AllUser).filter_by(name=username)
        if result.count():
            user = result.first()
            user.last_login = datetime.datetime.now()
            if user.public_key != key:
                user.public_key = key
        else:
            raise ValueError(f'Пользователь не зарегистрирован')

        new_active_user = self.ActiveUser(
            user.id,
            ip_address,
            port,
            datetime.datetime.now()
        )
        self.session.add(new_active_user)

        self.LoginHistory(
            user.id,
            ip_address,
            port,
            datetime.datetime.now()
        )
        self.session.add(new_active_user)
        self.session.commit()

    def add_user(self, username, password_hash):
        user = self.AllUser(username, password_hash)
        self.session.add(user)
        self.session.commit()
        user_in_history = self.UserHistory(user.id)
        self.session.add(user_in_history)
        self.session.commit()

    def remove_user(self, username):
        user = self.session.query(self.AllUser).filter_by(
            name=username).first()
        self.session.query(self.LoginHistory).filter_by(user=user.id).delete()
        self.session.query(self.ActiveUser).filter_by(user=user.id).delete()
        self.session.query(self.UserContacts).filter_by(user=user.id).delete()
        self.session.query(self.UserHistory).filter_by(user=user.id).delete()
        self.session.query(self.AllUser).filter_by(id=user.id).delete()

        self.session.commit()

    def get_password_hash(self, username):
        user = self.session.query(self.AllUser).filter_by(
            name=username).first()
        return user.password_hash

    def get_public_key(self, username):
        user = self.session.query(self.AllUser).filter_by(
            name=username).first()
        return user.public_key

    def check_user(self, username):
        user = self.session.query(self.AllUser).filter_by(
            name=username).count()
        if user:
            return True
        else:
            return False

    def user_logout(self, username):
        user = self.session.query(self.AllUser).filter_by(
            name=username).first()
        self.session.query(self.ActiveUser).filter_by(user=user.id).delete()
        self.session.commit()

    def users_list(self):
        query = self.session.query(
            self.AllUser.name,
            self.AllUser.last_login
        ).all()
        return query

    def active_users_list(self):
        query = self.session.query(
            self.AllUser.name,
            self.ActiveUser.ip_address,
            self.ActiveUser.port,
            self.ActiveUser.at_login_time,
        ).join(self.AllUser).all()
        return query

    def login_history_list(self, username=None):
        query = self.session.query(
            self.AllUser.name,
            self.LoginHistory.ip_address,
            self.LoginHistory.port,
            self.LoginHistory.date,
        ).join(self.AllUser)
        if username:
            query = query.filter(self.AllUser.name == username)
        return query.all()

    def increment_sent_accepted_message(self, sender, recipient):
        sender_id = self.session.query(self.AllUser).filter_by(
            name=sender).first().id
        recipient_id = self.session.query(self.AllUser).filter_by(
            name=recipient).first().id
        sender_history = self.session.query(self.UserHistory).filter_by(
            user=sender_id).first()
        sender_history.sent += 1
        recipient_history = self.session.query(self.UserHistory).filter_by(
            user=recipient_id).first()
        recipient_history.accepted += 1
        self.session.commit()

    def add_contact(self, user, contact):
        user = self.session.query(self.AllUser).filter_by(name=user).first()
        contact = self.session.query(self.AllUser).filter_by(
            name=contact).first()
        if not contact or self.session.query(self.UserContacts).filter_by(
                user=user.id, contact=contact.id).count():
            return
        row = self.UserContacts(user.id, contact.id)
        self.session.add(row)
        self.session.commit()

    def remove_contact(self, user, contact):
        user = self.session.query(self.AllUser).filter_by(name=user).first()
        contact = self.session.query(self.AllUser).filter_by(
            name=contact).first()
        if not contact:
            return
        self.session.query(self.UserContacts).filter_by(
            user=user.id, contact=contact.id).delete()
        self.session.commit()

    def get_contacts(self, username):
        user = self.session.query(self.AllUser).filter_by(name=username).one()
        query = self.session.query(
            self.UserContacts.user,
            self.AllUser.name
        ).filter_by(user=user.id).join(
            self.AllUser,
            self.UserContacts.contact == self.AllUser.id
        )
        return [contact[1] for contact in query.all()]

    def message_history(self, username=None):
        query = self.session.query(
            self.AllUser.name,
            self.UserHistory.sent,
            self.UserHistory.accepted
        ).join(self.AllUser)
        if username:
            query = query.filter_by(name=username)
        return query.all()



import logging
import sys

if sys.argv[0].split("/")[-1] == 'client.py':
    LOGGER = logging.getLogger('client')
elif sys.argv[0].split("/")[-1] == 'server_start.py':
    LOGGER = logging.getLogger('server')


def log(func):
    def log_saver(*args, **kwargs):
        result = func(*args, **kwargs)
        LOGGER.debug(
            f'DECORATED '  # для удобства отладки
            f'Вызвана функция {func.__name__} с параметрами {args, kwargs} '
            f'из модуля {func.__module__}. ',
            stacklevel=2
        )
        return result
    return log_saver



import argparse
import configparser
import os
import sys
import threading

from PyQt5.QtWidgets import QApplication

from server.common.variables import DEFAULT_PORT
from server.main.core import MessageProcessor
from server.main.database import ServerStorage
from server.gui.main_window import MainWindow

database_lock = threading.Lock()
new_connection = False
conflag_lock = threading.Lock()


def parse_arg(default_port, default_address):
    parser = argparse.ArgumentParser()
    parser.add_argument('-a', default=default_address, nargs='?')
    parser.add_argument('-p', default=default_port, type=int, nargs='?')
    namespace = parser.parse_args(sys.argv[1:])
    listen_address = namespace.a
    listen_port = namespace.p
    return listen_address, listen_port


def config_load():
    config = configparser.ConfigParser()
    dir_path = os.getcwd()
    config.read(f'server.ini')
    # Если конфиг файл загружен правильно, запускаемся,
    # иначе конфиг по умолчанию.
    if 'SETTINGS' in config:
        return config
    else:
        config.add_section('SETTINGS')
        config.set('SETTINGS', 'Default_port', str(DEFAULT_PORT))
        config.set('SETTINGS', 'Listen_Address', '')
        config.set('SETTINGS', 'Database_path', dir_path)
        config.set('SETTINGS', 'Database_file', 'server_database.db3')
        return config


def main():
    config = config_load()
    listen_address, listen_port = parse_arg(
        config['SETTINGS']['Default_port'],
        config['SETTINGS']['Listen_Address']
    )
    database = ServerStorage(
        os.path.join(
            config['SETTINGS']['Database_path'],
            config['SETTINGS']['Database_file']
        )
    )

    server = MessageProcessor(database, listen_address, listen_port)
    server.daemon = True
    server.start()

    server_app = QApplication(sys.argv)
    main_window = MainWindow(database, server, config)

    server_app.exec_()
    server.running = False


if __name__ == '__main__':
    main()



import ipaddress
import sys


class ValidateAddress:
    def __set__(self, instance, value):
        try:
            if value != '':
                validate_ip_address = ipaddress.ip_address(value)
        except ValueError:
            print('Введен некорректный IP адрес')
            sys.exit(1)
        else:
            instance.__dict__[self.my_attr] = value

    def __set_name__(self, owner, name):
        self.my_attr = name


class ValidatePort:
    def __set__(self, instance, value):
        if value < 1024 or value > 65535:
            print('Необходимо использовать порт от 1024 до 65535')
            sys.exit(1)
        instance.__dict__[self.my_attr] = value

    def __set_name__(self, owner, name):
        self.my_attr = name



from PyQt5.QtCore import QTimer
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from PyQt5.QtWidgets import QMainWindow, QAction, qApp, QLabel, \
    QTableView

from server.gui.add_user import RegisterUser
from server.gui.config_window import ConfigWindow
from server.gui.remove_user import DelUserDialog
from server.gui.stat_window import StatWindow


class MainWindow(QMainWindow):
    def __init__(self, database, server, config):
        super(MainWindow, self).__init__()
        self.database = database
        self.server_thread = server
        self.config = config
        exitAction = QAction('Выход', self)
        exitAction.setShortcut('Ctrl+Q')
        exitAction.triggered.connect(qApp.quit)
        self.refresh_button = QAction('Обновить список', self)
        self.config_btn = QAction('Настройки сервера', self)
        self.register_btn = QAction('Регистрация пользователя', self)
        self.remove_btn = QAction('Удаление пользователя', self)
        self.show_history_button = QAction('История клиентов', self)
        self.statusBar()
        self.toolbar = self.addToolBar('MainBar')
        self.toolbar.addAction(exitAction)
        self.toolbar.addAction(self.refresh_button)
        self.toolbar.addAction(self.show_history_button)
        self.toolbar.addAction(self.config_btn)
        self.toolbar.addAction(self.register_btn)
        self.toolbar.addAction(self.remove_btn)
        self.setFixedSize(800, 600)
        self.setWindowTitle('alpha')

        self.label = QLabel('Список клиентов:', self)
        self.label.setFixedSize(240, 15)
        self.label.move(15, 35)
        self.label = QLabel('Список подключённых клиентов:', self)
        self.label.setFixedSize(240, 15)
        self.label.move(305, 35)
        self.all_clients_table = QTableView(self)
        self.all_clients_table.move(10, 55)
        self.all_clients_table.setFixedSize(150, 400)
        self.active_clients_table = QTableView(self)
        self.active_clients_table.move(300, 55)
        self.active_clients_table.setFixedSize(400, 400)

        timer = QTimer()
        timer.timeout.connect(self.gui_create_model_active_users)
        timer.timeout.connect(self.gui_create_model_all_users)
        timer.start(1000)

        # Связываем кнопки с процедурами
        self.refresh_button.triggered.connect(
            self.gui_create_model_active_users)
        self.show_history_button.triggered.connect(self.show_statistics)
        self.config_btn.triggered.connect(self.server_config)
        self.register_btn.triggered.connect(self.reg_user)
        self.remove_btn.triggered.connect(self.rem_user)
        self.show()

    def gui_create_model_active_users(self):
        """Создание модели активных пользователей"""
        list_users = self.database.active_users_list()
        lst = QStandardItemModel()
        lst.setHorizontalHeaderLabels(
            ['Имя Клиента', 'IP Адрес', 'Порт', 'Время подключения'])
        for row in list_users:
            user, ip, port, time = row
            user = QStandardItem(user)
            user.setEditable(False)
            ip = QStandardItem(ip)
            ip.setEditable(False)
            port = QStandardItem(str(port))
            port.setEditable(False)
            time = QStandardItem(str(time.replace(microsecond=0)))
            time.setEditable(False)
            lst.appendRow([user, ip, port, time])
        self.active_clients_table.setModel(lst)
        self.active_clients_table.resizeColumnsToContents()
        self.active_clients_table.resizeRowsToContents()

    def gui_create_model_all_users(self):
        """Создание модели всех пользователей"""
        list_users = self.database.users_list()
        lst = QStandardItemModel()
        lst.setHorizontalHeaderLabels(['Имя Клиента'])
        for row in list_users:
            user = row[0]
            user = QStandardItem(user)
            user.setEditable(False)
            lst.appendRow([user])
        self.all_clients_table.setModel(lst)
        self.all_clients_table.resizeColumnsToContents()
        self.all_clients_table.resizeRowsToContents()

    def show_statistics(self):
        """Метод создающий окно со статистикой клиентов."""
        global stat_window
        stat_window = StatWindow(self.database)
        stat_window.show()

    def server_config(self):
        """Метод создающий окно с настройками сервера."""
        global config_window
        # Создаём окно и заносим в него текущие параметры
        config_window = ConfigWindow(self.config)

    def reg_user(self):
        """Метод создающий окно регистрации пользователя."""
        global reg_window
        reg_window = RegisterUser(self.database, self.server_thread)
        reg_window.show()

    def rem_user(self):
        """Метод создающий окно удаления пользователя."""
        global rem_window
        rem_window = DelUserDialog(self.database, self.server_thread)
        rem_window.show()



import logging

DEFAULT_PORT = 7880
DEFAULT_IP_ADDRESS = '127.0.0.1'
MAX_CONNECTIONS = 5
MAX_PACKAGE_LENGTH = 10240
ENCODING = 'utf-8'

ACTION = 'action'
TIME = 'time'
USER = 'user'
SENDER_USER = 'from'
DESTINATION_USER = 'to'
ACCOUNT_NAME = 'account_name'
EXIT = 'exit'

PRESENCE = 'presence'
MESSAGE = 'message'
ADD_CONTACT = 'add_contact'
USER_TO_ADDED = 'user_to_added'
REMOVE_CONTACT = 'remove_contact'
GET_CONTACTS = 'get_contacts'
GET_ALL_USERS = 'get_users_list'
LIST_INFO = 'contact_list'
MESSAGE_TEXT = 'message_text'
RESPONSE = 'response'
ERROR = 'error'
DATA = 'data'
PUBLIC_KEY = 'public_key'
PUBLIC_KEY_REQUEST = 'public_key_request'

RESPONSE_200 = {
    RESPONSE: 200
}
RESPONSE_205 = {
    RESPONSE: 205
}
RESPONSE_400 = {
    RESPONSE: 400
}
RESPONSE_511 = {
    RESPONSE: 511
}

LOGGING_LEVEL = logging.DEBUG

SERVER_DATABASE = f'sqlite:///server_database.db'



import binascii
import hmac
import json
import logging
import os
import select
import socket
import threading

from server.common.descriptors import ValidatePort, ValidateAddress
from server.common.utils import get_message, send_message
from server.common.variables import ACTION, PRESENCE, TIME, ACCOUNT_NAME, ERROR, \
    MESSAGE, MESSAGE_TEXT, SENDER_USER, EXIT, DESTINATION_USER, \
    ADD_CONTACT, RESPONSE_200, RESPONSE_400, REMOVE_CONTACT, GET_CONTACTS, \
    LIST_INFO, USER, GET_ALL_USERS, \
    MAX_CONNECTIONS, RESPONSE_511, DATA, RESPONSE, PUBLIC_KEY, RESPONSE_205, \
    PUBLIC_KEY_REQUEST

logger = logging.getLogger('server')

database_lock = threading.Lock()
message = None
new_connection = False
conflag_lock = threading.Lock()


class MessageProcessor(threading.Thread):
    listen_port = ValidatePort()
    listen_address = ValidateAddress()

    def __init__(self, database, listen_address, listen_port):
        self.server_socket = None
        self.listen_address = listen_address
        self.listen_port = listen_port
        self.database = database
        self.clients_list = []
        self.message_list = []
        self.users_list = {}
        self.listen_socket = None
        self.error_socket = None
        self.running = True

        super().__init__()

    def init_socket(self):
        logger.info(
            f'Запущен сервер, порт для подключений: {self.listen_port}, адрес'
            f' с которого принимаются подключения: {self.listen_address}.'
            f' Если адрес не указан, принимаются соединения с любых адресов.')

        transport = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        transport.bind((self.listen_address, self.listen_port))
        transport.settimeout(0.5)
        self.server_socket = transport
        self.server_socket.listen(MAX_CONNECTIONS)

    def run(self):
        global message
        global new_connection
        self.init_socket()
        while self.running:
            try:
                client, client_address = self.server_socket.accept()
            except OSError:
                pass
            else:
                logger.info(
                    f'Установлено соединение с клиентом {client_address}')
                client.settimeout(5)
                self.clients_list.append(client)

            read_data_list = []
            try:
                if self.clients_list:
                    read_data_list, self.listen_socket, self.error_socket = select.select(
                        self.clients_list,
                        self.clients_list,
                        [], 0)
            except OSError as error:
                logger.error(f'Ошибка работы с сокетами: {error.errno}')

            if read_data_list:
                for client_with_message in read_data_list:
                    try:
                        message = get_message(client_with_message)
                        self.process_client_message(message,
                                                    client_with_message)
                    except (OSError, json.JSONDecodeError, TypeError) as error:
                        logger.debug(
                            'Getting data from client exception.',
                            exc_info=error
                        )
                        self.remove_client(client_with_message)

    def remove_client(self, client):
        logger.info(f'Клиент {client.getpeername()} отключился от сервера.')
        for name in self.users_list:
            if self.users_list[name] == client:
                self.database.user_logout(name)
                del self.users_list[name]
                break
        self.clients_list.remove(client)
        client.close()

    def check_user_and_send_message(self, message):
        if message[DESTINATION_USER] in self.users_list and \
                self.users_list[
                    message[DESTINATION_USER]] in self.listen_socket:
            try:
                print('message - ', message)
                print(self.users_list)
                send_message(message,
                             self.users_list[message[DESTINATION_USER]])
                logger.info(
                    f'Отправлено сообщение пользователю'
                    f' {message[DESTINATION_USER]} от пользователя'
                    f' {message[SENDER_USER]}.'
                )
            except OSError:
                self.remove_client(message[DESTINATION_USER])
        elif message[DESTINATION_USER] in self.users_list and \
                self.users_list[
                    message[DESTINATION_USER]] not in self.listen_socket:
            logger.error(
                f'Связь с клиентом {message[DESTINATION_USER]} была потеряна.'
                f' Соединение закрыто, доставка невозможна.'
            )
            self.remove_client(self.users_list[message[DESTINATION_USER]])
        else:
            logger.error(
                f'Пользователь {message[DESTINATION_USER]} '
                f'не зарегистрирован на сервере.'
            )

    def process_client_message(self, message, client):
        """Обработчик сообщений от клиентов"""
        logger.debug(f'Разбор сообщения от клиента : {message}')
        # Сообщение о присутствии
        if ACTION in message and message[ACTION] == PRESENCE \
                and TIME in message and ACCOUNT_NAME in message:
            self.autorize_user(message, client)
        # Сообщение с текстом
        elif ACTION in message and message[ACTION] == MESSAGE and \
                TIME in message and MESSAGE_TEXT in message:
            if message[DESTINATION_USER] in self.users_list:
                self.check_user_and_send_message(message)
                self.database.increment_sent_accepted_message(
                    message[SENDER_USER], message[DESTINATION_USER])
                try:
                    send_message(RESPONSE_200, client)
                except OSError:
                    self.remove_client(client)
            else:
                response = RESPONSE_400
                response[ERROR] = 'Пользователь не зарегистрирован на сервере'
                try:
                    send_message(response, client)
                except OSError:
                    pass
            return
        # Сообщение добавление контакта
        elif ACTION in message and message[ACTION] == ADD_CONTACT and \
                USER in message and ACCOUNT_NAME in message:
            self.database.add_contact(message[ACCOUNT_NAME], message[USER])
            try:
                send_message(RESPONSE_200, client)
            except OSError:
                self.remove_client(client)
            return
        # Сообщение удаление контакта
        elif ACTION in message and message[ACTION] == REMOVE_CONTACT and \
                USER in message and ACCOUNT_NAME in message:
            self.database.remove_contact(message[ACCOUNT_NAME], message[USER])
            try:
                send_message(RESPONSE_200, client)
            except OSError:
                self.remove_client(client)
            return
        # Сообщение запрос списка контактов
        elif ACTION in message and message[ACTION] == GET_CONTACTS and\
                ACCOUNT_NAME in message:
            response = RESPONSE_200
            response[LIST_INFO] = self.database.get_contacts(
                message[ACCOUNT_NAME])
            try:
                send_message(response, client)
            except OSError:
                self.remove_client(client)
            return
        # Сообщение запрос списка контактов
        elif ACTION in message and message[ACTION] == GET_ALL_USERS:
            response = RESPONSE_200
            response[LIST_INFO] = [user[0] for user in
                                   self.database.users_list()]
            try:
                send_message(response, client)
            except OSError:
                self.remove_client(client)
            return
        # Сообщение запрос на выход
        elif ACTION in message and message[ACTION] == EXIT and\
                ACCOUNT_NAME in message:
            self.remove_client(client)
        elif ACTION in message and message[ACTION] == PUBLIC_KEY_REQUEST and\
                ACCOUNT_NAME in message:
            response = RESPONSE_511
            response[DATA] = self.database.get_public_key(
                message[ACCOUNT_NAME])
            if response[DATA]:
                try:
                    send_message(response, client)
                except OSError:
                    self.remove_client(client)
            else:
                response = RESPONSE_400
                response[
                    ERROR] = 'Нет публичного ключа для данного пользователя'
                try:
                    send_message(response, client, )
                except OSError:
                    self.remove_client(client)
        else:
            send_message(RESPONSE_400, client)

    def autorize_user(self, message, client):
        logger.debug(f'Запущен процесс авторизации {message[ACCOUNT_NAME]}')
        if message[ACCOUNT_NAME] in self.users_list.keys():
            response = RESPONSE_400
            response[ERROR] = 'Имя пользователя уже занято'
            try:
                send_message(response, client)
            except OSError:
                pass
            else:
                self.clients_list.remove(client)
                client.close()

        elif not self.database.check_user(message[ACCOUNT_NAME]):
            response = RESPONSE_400
            response[ERROR] = 'Пользователь не зарегистрирован'
            try:
                send_message(response, client)
            except OSError:
                pass
            else:
                self.clients_list.remove(client)
                client.close()
        else:
            logger.debug(
                'Пользователь корректный, начинается проверка пароля')

        message_auth = RESPONSE_511
        random_str = binascii.hexlify(os.urandom(64))
        message_auth[DATA] = random_str.decode('ascii')
        hash_password = hmac.new(
            self.database.get_password_hash(message[ACCOUNT_NAME]),
            random_str,
            'MD5'
        )
        print(self.database.get_password_hash(message[ACCOUNT_NAME]))
        digest = hash_password.digest()
        logger.debug(f'сообщение авторизации = {message_auth}')
        try:
            send_message(message_auth, client)
            ans = get_message(client)
        except OSError as error:
            logger.debug('Ошибка аутентификации', exc_info=error)
            client.close()
            return
        client_digest = binascii.a2b_base64(ans[DATA])
        print(digest, client_digest)
        if RESPONSE in ans and ans[RESPONSE] == 511 and hmac.compare_digest(
                digest, client_digest):
            self.users_list[message[ACCOUNT_NAME]] = client
            client_ip, client_port = client.getpeername()
            try:
                send_message(RESPONSE_200, client)
            except OSError:
                self.remove_client(client)
            self.database.user_login(message[ACCOUNT_NAME], client_ip,
                                     client_port, message[PUBLIC_KEY])

        else:
            response = RESPONSE_400
            response[ERROR] = 'Неверный пароль.'
            try:
                send_message(response, client)
            except OSError:
                pass
            self.clients_list.remove(client)
            client.close()

    def service_update_lists(self):
        for client in self.users_list:
            try:
                send_message(self.users_list[client], RESPONSE_205)
            except OSError:
                self.remove_client(self.users_list[client])


if __name__ == '__main__':
    pass



import sys
from cx_Freeze import setup, Executable

build_exe_options = {
    "packages": ["common", "project_logs", "gui", 'main'],
}
setup(
    name="mess_server",
    version="0.0.1",
    description="mess_server",
    options={
        "build_exe": build_exe_options
    },
    executables=[Executable('main/server_start.py',
                            # base='Win32GUI',
                            targetName='server_start.exe',
                            )]
)



from setuptools import setup, find_packages

setup(name="vyatkin_chat_server",
      version="0.0.1",
      description="vyatkin_chat_server",
      author="Vyatkin M.",
      author_email="",
      packages=find_packages(),
      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )



"""Утилиты"""

import json

from server.common.variables import ENCODING, MAX_PACKAGE_LENGTH


def get_message(client):
    encoded_response = client.recv(MAX_PACKAGE_LENGTH)
    if isinstance(encoded_response, bytes):
        json_response = encoded_response.decode(ENCODING)
        response = json.loads(json_response)
        if isinstance(response, dict):
            return response
        else:
            raise TypeError


def send_message(message, socket):
    js_message = json.dumps(message)
    encoded_message = js_message.encode(ENCODING)
    socket.send(encoded_message)



import dis


def dis_clsdict(clsdict):
    methods = []
    attrs = []
    for func in clsdict:
        try:
            ret = dis.get_instructions(clsdict[func])
        except:
            pass
        else:
            for i in ret:
                if i.opname == 'LOAD_METHOD':
                    if i.argval not in methods:
                        methods.append(i.argval)
                if i.opname == 'LOAD_ATTR':
                    if i.argval not in attrs:
                        attrs.append(i.argval)
    return methods, attrs


def validate_method_and_attr(clsname, methods, attrs):
    if not ('SOCK_STREAM' in attrs and 'AF_INET' in attrs):
        raise TypeError('Некорректная инициализация сокета')
    if clsname == 'Client':
        if 'listen' in methods or 'accept' in methods:
            raise TypeError(
                'Использование методов "listen"'
                ' или "accept" недопустимо в клиентском классе')
    elif clsname == 'Server':
        if 'connect' in methods:
            raise TypeError('Использование методов "connect" '
                            'недопустимо в серверном классе')


class ClientMaker(type):

    def __init__(self, clsname, bases, clsdict):
        methods, attrs = dis_clsdict(clsdict)
        validate_method_and_attr(clsname, methods, attrs)
        super().__init__(clsname, bases, clsdict)


class ServerMaker(type):

    def __init__(self, clsname, bases, clsdict):
        methods, attrs = dis_clsdict(clsdict)
        validate_method_and_attr(clsname, methods, attrs)
        super().__init__(clsname, bases, clsdict)
